#
#      heatmap_generator.tcl
#      =====================
#        Uwe Berger; 2012
#
package require sqlite3

set tab_temp			"temp_log"
set db_temp				"/home/bergeruw/work/owfs/owfs_log"
set hm_file_temperature	"/home/bergeruw/public_html/temp/heatmap_temperature.png"

set tab_light			"adc_log"
set db_light			"/home/bergeruw/work/i2c/i2c_log"
set hm_file_light		"/home/bergeruw/public_html/temp/heatmap_light.png"

set tab_humidity		"humidity_log_neu"
set db_humidity			"/home/bergeruw/work/i2c/i2c_log"
set hm_file_humidity	"/home/bergeruw/public_html/temp/heatmap_humidity.png"


# Intervalle in Sekunden
set day_sec				86400
set day_interval		300
set dx_sec				[expr 365 * $day_sec]

set gvar(gnuplot_file)	"data.txt"
set gvar(size_x)		900
set gvar(size_y)		400

# *********************************
proc dt_convert {dt_sec} {
	return [clock format $dt_sec -gmt 1 -format %Y-%m-%dT%H:%M:%S]
}

# *********************************
proc d_convert {dt_sec} {
	return [clock format $dt_sec -gmt 1 -format %Y-%m-%d]
}

# *********************************
proc write_gnuplot_input {gf y z from} {
	global day_interval
	set date [clock format $from -gmt 1 -format %Y-%m-%d]
	set y [format "%.2f" [expr $y * ($day_interval/60)/60.0]]
	set z [format "%.2f" $z]
	puts $gf "$date $y $z"
}

# *********************************
proc heatmap_temperature {min_day} {
	global db_temp tab_temp gvar day_interval day_sec hm_file_temperature
	puts "Generate heatmap for temperature..."
	sqlite db $db_temp
	db timeout 2000
	#set sql "select strftime('%s', strftime('%Y-%m-%d', min(ts))) from $tab_temp"
	#set min_day [db onecolumn $sql]
	set sql "select strftime('%s', strftime('%Y-%m-%d', max(ts))) from $tab_temp"
	set max_day [db onecolumn $sql]
	set from_p [d_convert $min_day]
	set to_p [d_convert $max_day]

	# Schleife ueber alle Tage zwischen Min und Max; gnuplot-File schreiben...
	puts [time {
	set gf [open $gvar(gnuplot_file) w]
	set day $min_day
	while {$day < [expr $max_day + $day_sec]} {
		set day_int $day
		set y 0
		while {$day_int < [expr $day + $day_sec]} {
			set from [dt_convert $day_int]
			set to [dt_convert [expr $day_int + $day_interval - 1]]
			# *********************************
			# ** '10405FB5010800B9' (aussen) **
			# ** '107D6EB501080057' (innen)  **
			# *********************************
			set sql "select avg(temp_log.'10405FB5010800B9') from $tab_temp where ts between '$from' and '$to'"
			set avg [db onecolumn $sql]
			if {![llength $avg]} {set avg 1000}
			write_gnuplot_input $gf $y $avg $day_int
			incr day_int $day_interval
			incr y
		}
		incr day $day_sec
	}
	close $gf
	}]
	db close
	# gnuplot aufrufen...
	set pid [open "| gnuplot" "w"]	
	puts $pid "unset key"
	puts $pid "set terminal png font arial 8 size $gvar(size_x), $gvar(size_y)"
	puts $pid "set view map"
	puts $pid "set output '$hm_file_temperature'"
	puts $pid "set title \"Temperaturverlauf über Datum/Tageszeit\\nvon $from_p bis $to_p\""
	puts $pid "set xdata time"
	puts $pid "set timefmt '%Y-%m-%d'"
	puts $pid "set format x \"%d.%m.\\n%Y\""
	puts $pid "set xrange \[\"$from_p\":\"$to_p\"\]"
	puts $pid "set xlabel 'Datum'"
	puts $pid "set yrange \[0:24\]"
	puts $pid "set ytics 1"
	puts $pid "set ylabel 'Tagesstunde'"
	puts $pid "set cblabel '°C'"
	puts $pid "set cbrange \[-25.0:50.0\]"
	puts $pid "set palette defined (0 \"blue\",17 \"#00ffff\",33 \"white\",55 \"yellow\",\\"
    puts $pid "	70 \"red\",100 \"#990000\", 101 \"grey\")"
	puts $pid "plot '$gvar(gnuplot_file)' using 1:2:3 with image"
	puts $pid ""
	close $pid
}

# *********************************
proc heatmap_light {min_day} {
	global db_light tab_light gvar day_interval day_sec hm_file_light
	puts "Generate heatmap for light..."
	sqlite db $db_light
	db timeout 2000
	#set sql "select strftime('%s', strftime('%Y-%m-%d', min(ts))) from $tab_light"
	#set min_day [db onecolumn $sql]
	set sql "select strftime('%s', strftime('%Y-%m-%d', max(ts))) from $tab_light"
	set max_day [db onecolumn $sql]
	set from_p [d_convert $min_day]
	set to_p [d_convert $max_day]

	# Schleife ueber alle Tage zwischen Min und Max; gnuplot-File schreiben...
	puts [time {
	set gf [open $gvar(gnuplot_file) w]
	set day $min_day
	while {$day < [expr $max_day + $day_sec]} {
		set day_int $day
		set y 0
		while {$day_int < [expr $day + $day_sec]} {
			set from [dt_convert $day_int]
			set to [dt_convert [expr $day_int + $day_interval - 1]]
			# ****************************
			# ** adc0 LDR
			# ** adc1 Solarzelle
			# ****************************
			set sql "select avg(adc0) from $tab_light where ts between '$from' and '$to'"
			set avg [db onecolumn $sql]
			if {![llength $avg]} {set avg 0}
			write_gnuplot_input $gf $y $avg $day_int
			incr day_int $day_interval
			incr y
		}
		incr day $day_sec
	}
	close $gf
	}]
	db close
	# gnuplot aufrufen...
	set pid [open "| gnuplot" "w"]	
	puts $pid "unset key"
	puts $pid "set terminal png font arial 8 size $gvar(size_x), $gvar(size_y)"
	puts $pid "set view map"
	puts $pid "set output '$hm_file_light'"
	puts $pid "set title \"Helligkeitsverlauf über Datum/Tageszeit\\nvon $from_p bis $to_p\""
	puts $pid "set xdata time"
	puts $pid "set timefmt '%Y-%m-%d'"
	puts $pid "set format x \"%d.%m.\\n%Y\""
	puts $pid "set xrange \[\"$from_p\":\"$to_p\"\]"
	puts $pid "set xlabel 'Datum'"
	puts $pid "set yrange \[0:24\]"
	puts $pid "set ytics 1"
	puts $pid "set ylabel 'Tagesstunde'"
	puts $pid "set cblabel 'Helligkeitswert'"
	puts $pid "set cbrange \[0:255\]"
	puts $pid "set palette defined (0 'black', 100 'white')"
	puts $pid "plot '$gvar(gnuplot_file)' using 1:2:3 with image"
	puts $pid ""
	close $pid
}

# *********************************
proc heatmap_humidity {min_day} {
	global db_humidity tab_humidity gvar day_interval day_sec hm_file_humidity
	puts "Generate heatmap for humidity..."
	sqlite db $db_humidity
	db timeout 2000
	#set sql "select strftime('%s', strftime('%Y-%m-%d', min(ts))) from $tab_humidity"
	#set min_day [db onecolumn $sql]
	set sql "select strftime('%s', strftime('%Y-%m-%d', max(ts))) from $tab_humidity"
	set max_day [db onecolumn $sql]
	set from_p [d_convert $min_day]
	set to_p [d_convert $max_day]

	# Schleife ueber alle Tage zwischen Min und Max; gnuplot-File schreiben...
	puts [time {
	set gf [open $gvar(gnuplot_file) w]
	set day $min_day
	while {$day < [expr $max_day + $day_sec]} {
		set day_int $day
		set y 0
		while {$day_int < [expr $day + $day_sec]} {
			set from [dt_convert $day_int]
			set to [dt_convert [expr $day_int + $day_interval - 1]]
			# ****************************
			# ** ((((adc/255.0)*5)-0.958062)*30.680)/(1.0546-(0.00216*temp))
			# ****************************
			set sql "select avg(((((adc/255.0)*5)-0.958062)*30.680)/(1.0546-(0.00216*temp))) from $tab_humidity where ts between '$from' and '$to'"
			set avg [db onecolumn $sql]
			if {![llength $avg]} {set avg 1000}
			write_gnuplot_input $gf $y $avg $day_int
			incr day_int $day_interval
			incr y
		}
		incr day $day_sec
	}
	close $gf
	}]
	db close
	# gnuplot aufrufen...
	set pid [open "| gnuplot" "w"]	
	puts $pid "unset key"
	puts $pid "set terminal png font arial 8 size $gvar(size_x), $gvar(size_y)"
	puts $pid "set view map"
	puts $pid "set output '$hm_file_humidity'"
	puts $pid "set title \"Verlauf der relativen Luftfeuchtigkeit über Datum/Tageszeit\\nvon $from_p bis $to_p\""
	puts $pid "set xdata time"
	puts $pid "set timefmt '%Y-%m-%d'"
	puts $pid "set format x \"%d.%m.\\n%Y\""
	puts $pid "set xrange \[\"$from_p\":\"$to_p\"\]"
	puts $pid "set xlabel 'Datum'"
	puts $pid "set yrange \[0:24\]"
	puts $pid "set ytics 1"
	puts $pid "set ylabel 'Tagesstunde'"
	puts $pid "set cblabel '%'"
	puts $pid "set cbrange \[0:100\]"
	puts $pid "set palette defined (0 'white', 100 'blue', 101 'grey')"
	puts $pid "plot '$gvar(gnuplot_file)' using 1:2:3 with image"
	puts $pid ""
	close $pid
}

# *********************************
proc generate {} {
	global dx_sec day_sec
	set xmin_sec [expr [clock seconds] - $dx_sec]
	set min_day [clock format $xmin_sec -gmt 1 -format %Y-%m-%d]
	puts $min_day
	set min_day_sec [expr ($xmin_sec/$day_sec)*$day_sec] 
	puts [time {heatmap_temperature $min_day_sec}]
	puts [time {heatmap_light $min_day_sec}]
	puts [time {heatmap_humidity $min_day_sec}]
	# alle 2h...
	after 7200000 generate
}

# *********************************
# *********************************
# *********************************
# *********************************

generate

# Endlos
vwait forever

