#
# Uwe Berger; 2011
#
package require sqlite3

set ds1820_dir       "/home/bergeruw/mnt/ow/uncached/10.*"
set tab              "temp_log"
set dbas             "/home/bergeruw/work/owfs/owfs_log"
set temp_data        "/home/bergeruw/work/owfs/data.txt"

set plot_config_file "/home/bergeruw/public_html/temp/plot_config.tcl"
set plot_file        "/home/bergeruw/public_html/temp/temp_plot.png"

set g(xsize) 900
set g(ysize) 500
set g(font) "arial 8"
set g(ymin) -10
set g(ymax)  40
set g(dt)    24


#*************************************
proc read_file {fname} {
	set fd [open $fname "r"]
	set d [read $fd]		
	close $fd
	return [string trim $d]
}

#*************************************
proc owfs_ds1820_to_sqlite {dir} {
	global tab
	set sql   [list insert into $tab ]
	set sql_c [list ts] 
	set sql_v [list '[clock format [clock seconds] -format "%Y-%m-%dT%H:%M:%S"]' ]
	# jeden ds1820 abfragen
	foreach f [glob $dir] {
		if ([file isdirectory $f]) {
			set sensor [read_file "$f/address"]
			# eventuell Sensor-Spalte in Tabelle anlegen
			if {[lsearch [db eval [list pragma table_info($tab)]] $sensor] > -1} {
				# Spalte vorhanden...
			} else {
				db eval [list alter table $tab add column '$sensor' real]
			}
			set sql_c [linsert $sql_c end '$sensor'] 
			set sql_v [linsert $sql_v end [read_file "$f/temperature"]]
		}
	}
	# sql-Kommando zusammensetzen
	set sql [linsert $sql end ( [join $sql_c ","] ) values ( [join $sql_v ","] )]
	puts $sql
	db eval $sql
}

#*************************************
proc task {} {
	global ds1820_dir
	owfs_ds1820_to_sqlite $ds1820_dir
	after 60000 task
}

#*****************
proc generate_plot {} {
	global g dbas tab plot_config_file temp_data plot_file
	# Versuch ein vorhandenes Konfig-File zu lesen
	catch {
		source $plot_config_file
	}
	puts "Diagramm zeichnen"

	# Timestamp ermitteln
	set day     [clock format [clock seconds] -format %d]
	set month   [clock format [clock seconds] -format %m]
	set year    [clock format [clock seconds] -format %Y]
	set hour    [clock format [clock seconds] -format %H]
	set minute  [clock format [clock seconds] -format %M]
	set seconds [clock format [clock seconds] -format %S]
	# xmin/ymin setzen
	set temp_d [clock scan "$year-$month-$day\T$hour:$minute:00" -format %Y-%m-%dT%H:%M:%S]
    	set g(xmax) "$year-$month-$day\T$hour:$minute:00"
   	set g(xmin) [clock format [expr $temp_d - $g(dt)*3600] -format %Y-%m-%dT%H:%M:%S]
	# temporaeres Datenfile erzeugen
	set sql "select * from $tab where ts > '$g(xmin)'"
	exec /usr/bin/sqlite3 $dbas $sql > $temp_data
	# gnuplot oeffnen...	
	set pid [open "| gnuplot" "w"]	
	puts $pid "set title \"Temperaturverlauf über $g(dt) Stunden\\naktuell: $day.$month.$year $hour:$minute:$seconds\""
	puts $pid "set size 1, 1"
	puts $pid "set ytics 5"
	puts $pid "set xdata time"
	puts $pid "set timefmt '%Y-%m-%dT%H:%M:%S'"
	puts $pid "set format x \"%H.%M\\n%d.%m\""
	puts $pid "set autoscale y"
	puts $pid "set autoscale x"
	puts $pid "set xrange \['$g(xmin)':'$g(xmax)'\]"
	puts $pid "set yrange \[$g(ymin):$g(ymax)\]"
	puts $pid "set ylabel '°C'"
	puts $pid "set datafile separator '|'"
	puts $pid "set grid ytics xtics"
	puts $pid "set terminal png font $g(font) size $g(xsize), $g(ysize)"
	puts $pid "set output '$plot_file'"
	puts -nonewline $pid "plot "
	puts -nonewline $pid "'$temp_data' using 1:3 title 'aussen' with lines, "
	puts -nonewline $pid "'$temp_data' using 1:2 title 'innen'  with lines "
	puts $pid ""
	close $pid
	after 120000 generate_plot
}

#***************************************
#***************************************
#***************************************

# Datenbank oeffnen bzw. eventuell neu anlegen
sqlite3 db $dbas
db timeout 250000

# gibt es Tabelle $tab
if {[db exists {select 1 from sqlite_master where name=$tab and type='table'}]} {
	# alles klar, Tabelle vorhanden
	puts "Tabelle vorhanden!" 
} else {
	# Tabelle anlegen
	db eval [list create table $tab (ts text)]
}

# Temperatur loggen
task

# Diagramm zeichnen
generate_plot

vwait forever
